import axios from "axios";
import { AuthOptions } from "next-auth";
import { JWT } from "next-auth/jwt";
import CredentialsProvider from "next-auth/providers/credentials";
import { apiInstance } from "./apiConfig";
import { IRefresh, ITokens } from "../types/token-type";
import GoogleProvider from "next-auth/providers/google";

const verifyToken = async (access_token: string) => {
  const res = await apiInstance.post("/token/verify/", {
    token: access_token,
  });

  if ([200, 201].includes(res.status)) {
    return true;
  }

  return false;
};

const refreshToken = async (token: JWT): Promise<JWT> => {
  const data = {
    refresh: token.refresh_token,
  };
  // const date = new Date().toLocaleTimeString();
  // const expire = new Date(token.expires_in).toLocaleTimeString();

  const verify = await verifyToken(token.access_token);

  if (verify)
    return {
      ...token,
    };

  const response = await apiInstance.post<IRefresh>(
    "/users/refresh/",
    data
  );

  return {
    ...token,
    access_token: response.data.access,
    expires_in: response.data.expires_at,
  };
};

export const authConfig: AuthOptions = {
  providers: [
    CredentialsProvider({
      name: "Credentials",
      credentials: {
        email: {
          label: "Email",
          type: "text",
          placeholder: "jsmith@example.com",
        },
        password: { label: "Password", type: "password" },
      },
      async authorize(credentials, req) {
        if (!credentials?.email || !credentials?.password)
          return null;

        const { email, password } = credentials as any;
        const data = {
          email,
          password,
        };

        const res = await apiInstance.post("/users/login/", data);

        if (![200, 201].includes(res.status)) {
          return null;
        }

        const user = res.data;
        return user;
      },
    }),
    GoogleProvider({
      clientId: process.env.CLIENT_ID as string,
      clientSecret: process.env.CLIENT_SECRET as string,
    }),
  ],
  pages: {
    signIn: "/sign-in",
  },
  session: {
    strategy: "jwt",
  },
  callbacks: {
    async signIn({ account, profile, user }) {
      if (!profile?.email) {
        throw new Error("No Profile");
      }

      if (account?.provider === "google") {
        const data = {
          auth_token: account?.id_token,
        };

        const res = await apiInstance.post<ITokens>(
          "/users/google/",
          data
        );

        if (![200, 201].includes(res.status)) {
          return false;
        }

        user.access_token = res.data.access_token;
        user.refresh_token = res.data.refresh_token;
        user.expires_in = res.data.expires_in;
      }

      return true;
    },
    async jwt({ token, user, account }) {
      if (user) return { ...token, ...user };

      if (token.exp) {
        return token;
      }

      return refreshToken(token);
    },

    async session({ token, session }) {
      session.access_token = token.access_token;
      session.refresh_token = token.refresh_token;

      return session;
    },
  },
};
